/*
 * gtkpasman
 * Copyright (C) 2003 Cyril Bellot
 *
 * configread.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "config.h"

#define max_line_length 256

char *passwdfile;
char *sshcmd;
char *ftpcmd;
char *telnetcmd;

char *globalpasswd;
int decrypted_passwds;

int eyevisible, listvisible, rememberpos;

int zoomsize;
int xe, ye;
int xl, yl;



/*
 * config_read
 * 	Read config from rc file
 */
int
config_read ()
{
  gchar buffer[max_line_length];
  FILE *fp;

  /* default values */
  passwdfile = g_strdup (DEFAULT_GPASSMAN_FILE);
  sshcmd = g_strdup (DEFAULT_SSHCMD);
  ftpcmd = g_strdup (DEFAULT_FTPCMD);
  telnetcmd = g_strdup (DEFAULT_TELNETCMD);
  eyevisible = DEFAULT_EYEVISIBLE;
  listvisible = DEFAULT_LISTVISIBLE;
  rememberpos = DEFAULT_REMEMBERPOS;
  zoomsize = DEFAULT_ZOOMSIZE;
  xe = -1;
  ye = -1;
  xl = -1;
  yl = -1;

  /* start reading configuration file */
  if ((fp = fopen (GTKPASMAN_CONF_FILE, "rt")) == NULL) {
    g_free (passwdfile);
    passwdfile = g_strdup (DEFAULT_GPASSMAN_FILE);
    g_free (sshcmd);
    sshcmd = g_strdup (DEFAULT_SSHCMD);
    g_free (ftpcmd);
    ftpcmd = g_strdup (DEFAULT_FTPCMD);
    g_free (telnetcmd);
    telnetcmd = g_strdup (DEFAULT_TELNETCMD);
    return 0;
  }

  /* 1st line: passwd file */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (passwdfile);
    passwdfile = g_strdup (g_strchomp (buffer));
  }

  /* 2nd line: ssh command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (sshcmd);
    sshcmd = g_strdup (g_strchomp (buffer));
  }

  /* 3rd line: ftp command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (ftpcmd);
    ftpcmd = g_strdup (g_strchomp (buffer));
  }

  /* 4th line: telnet command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (telnetcmd);
    telnetcmd = g_strdup (g_strchomp (buffer));
  }

  /* 5th line: eye always visible? */
  if (fgets (buffer, max_line_length, fp) != NULL)
    eyevisible = !(strcmp (buffer, "1\n"));

  /* 6th line: list visible last time? */
  if (fgets (buffer, max_line_length, fp) != NULL)
    listvisible = !(strcmp (buffer, "1\n"));

  /* 7th line: remember positions? */
  if (fgets (buffer, max_line_length, fp) != NULL)
    rememberpos = !(strcmp (buffer, "1\n"));

  /* 8th line: zoomsize */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    if (!sscanf (buffer, "%d", &zoomsize)) {
      zoomsize = DEFAULT_ZOOMSIZE;
    }
  }

  /* 9th line: x coord. for eye */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    if (!sscanf (buffer, "%d", &xe)) {
      xe = -1;
    }
  }

  /* 10th line: y coord. for eye */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    if (!sscanf (buffer, "%d", &ye)) {
      ye = -1;
    }
  }

  /* 11th line: x coord. for list */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    if (!sscanf (buffer, "%d", &xl)) {
      xl = -1;
    }
  }

  /* 12th line: y coord. for list */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    if (!sscanf (buffer, "%d", &yl)) {
      yl = -1;
    }
  }

  fclose (fp);
  return 1;
}

/*
 * config_write
 * 	write configuration file
 */
int
config_write ()
{
  FILE *fp;
  int txe, tye, txl, tyl;

  if ((fp = fopen (GTKPASMAN_CONF_FILE, "w")) == NULL) {
    g_print ("Failed to write config...\n");
    return 0;
  }
  /* Positions to store */
  if (rememberpos) {
    txe = xe;
    tye = ye;
    txl = xl;
    tyl = yl;
  }
  else {
    txe = tye = txl = tyl = -1;
  }

  /* write to file */
  if (!fputs
      (g_strdup_printf
       ("%s\n%s\n%s\n%s\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d", passwdfile, sshcmd,
	ftpcmd, telnetcmd, eyevisible, listvisible, rememberpos, zoomsize,
	txe, tye, txl, tyl), fp)) {
    g_print ("Failed to write config...\n");
    return 0;
  }

  fclose (fp);
  return 1;
}
