/*
 * gtkpasman
 * Copyright (C) 2003 Cyril Bellot
 *
 * interface.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
#include "config.h"
#include "interface.h"
#include "passwd.h"
#include "configread.h"

/* picture for the eye */
#include "pixmaps/on.xpm"
#include "pixmaps/off.xpm"

static void change_display_state (void);
static int eye_clic (GtkWidget * widget,
		     GdkEventButton * event, gpointer data);
static void change_pass (GtkWidget * passtxt);
static void select_row (GtkCTree * ctree, GtkCTreeNode * node);
static void select_command (gchar ** data);
static void select_passwd (gchar ** data);
static void select_passwd_and_exec_command (gchar ** data);
static void button_copy_cmd_line (void);
static void button_copy_passwd (void);
static void button_copy_passwd_and_execute (void);
static void menu_copy_passwd (gpointer data);
static void menu_copy_passwd_and_execute (gpointer data);
static void create_gtkpasman_config (void);
static void change_config (GtkWidget * dialog);
static gchar *give_factory_path (gchar * customer,
				 gchar * group,
				 gchar * host,
				 gchar * service,
				 gchar * serviceplus, gchar * user);
static gchar *give_factory_path_shortcut (gchar * customer,
					  gchar * group,
					  gchar * host, gchar * servicecourt);
static void add_to_recent_menu (gchar * title);
static void empty_hosts_recent_menu (void);


/* widgets to be used in several places */
GtkWidget *gtkpasman1;
GtkWidget *gtkpasman2;
GtkWidget *current;
GdkPixmap *on;
GdkBitmap *mask_on;
GdkPixmap *off;
GdkBitmap *mask_off;
GtkItemFactory *item_factory1;
GtkItemFactory *item_factory2;

GtkWidget *tree1;
GtkWidget *clipboard;
GtkCTreeNode *selectednode;
GtkWidget *entry4;
GtkWidget *entry5;
GtkWidget *entry6;
GtkWidget *entry7;
GtkWidget *entry8;
GtkWidget *button4;
GtkWidget *button5;



/* history of commands/passwords */
#define MAXHISTORY      4
GtkItemFactoryEntry history[MAXHISTORY];

/* menu for list window */
static GtkItemFactoryEntry Menu1[] = {
  {"/_File", NULL, NULL, 0, "<Branch>"},
  {"/File/Open file", NULL, NULL, 0, "<Item>"},
  {"/File/Settings", NULL, create_gtkpasman_config, 0, "<Item>"},
  {"/File/Quit", NULL, gtk_main_quit, 0, "<Item>"},
  {"/Refresh", NULL, create_gtkpasman_askpassword, 0, "<Item>"},
  {"/Reduce", NULL, change_display_state, 0, "<Item>"},
};

/* menu for eye windows */
static GtkItemFactoryEntry Menu2[] = {
  {"/_File", NULL, NULL, 0, "<Branch>"},
  {"/File/Open file", NULL, NULL, 0, "<Item>"},
  {"/File/Settings", NULL, create_gtkpasman_config, 0, "<Item>"},
  {"/File/Quit", NULL, gtk_main_quit, 0, "<Item>"},
  {"/Refresh", NULL, create_gtkpasman_askpassword, 0, "<Item>"},
  {"/Expand-reduce", NULL, change_display_state, 0, "<Item>"},
  {"/Hosts", NULL, NULL, 0, "<Branch>"},
  {"/Recent", NULL, NULL, 0, "<Branch>"},
};

/*
 * create_gtkpasman_list
 *      create the list window and all the widgets
 */
GtkWidget *
create_gtkpasman_list (void)
{
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *menubar1;
  GtkWidget *scroll1;
  GtkAccelGroup *menu1_accels;

  /* main window */
  gtkpasman1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (gtkpasman1), "gtkpasman");
  gtk_window_set_wmclass (GTK_WINDOW (gtkpasman1),
			  "gtkpasmanMain", "gtkpasman");
  gtk_signal_connect (GTK_OBJECT (gtkpasman1), "delete_event",
		      (GtkSignalFunc) gtk_main_quit, NULL);
  gtk_signal_connect (GTK_OBJECT (gtkpasman1), "destroy_event",
		      (GtkSignalFunc) gtk_main_quit, NULL);
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (gtkpasman1), vbox);

  /* menus */
  menu1_accels = gtk_accel_group_new ();
  item_factory1 = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<Main>",
					menu1_accels);
  gtk_item_factory_create_items (item_factory1, 6, Menu1, NULL);
  gtk_accel_group_attach (menu1_accels, GTK_OBJECT (gtkpasman1));
  menubar1 = gtk_item_factory_get_widget (item_factory1, "<Main>");
  gtk_menu_item_right_justify (GTK_MENU_ITEM
			       (gtk_item_factory_get_widget
				(item_factory1, "/Reduce")));
  gtk_box_pack_start (GTK_BOX (vbox), menubar1, FALSE, FALSE, 0);

  /* scrollwindow */
  scroll1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start_defaults (GTK_BOX (vbox), scroll1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll1),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  /* tree */
  tree1 = gtk_ctree_new (1, 0);
  gtk_container_add (GTK_CONTAINER (scroll1), tree1);
  gtk_widget_set_usize (tree1, 50, 100);
  gtk_signal_connect (GTK_OBJECT (tree1), "tree_select_row",
		      (GtkSignalFunc) select_row, NULL);

  /* bottom buttons */
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
  button = gtk_button_new_with_label ("Copy\ncommand line");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) button_copy_cmd_line, NULL);
  button = gtk_button_new_with_label ("Copy passwd\nonly");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) button_copy_passwd, NULL);
  button = gtk_button_new_with_label ("Copy passwd\n& execute");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) button_copy_passwd_and_execute, NULL);

  /* show all */
  gtk_widget_show_all (gtkpasman1);
  if ((xl != -1) && (yl != -1))
    gtk_widget_set_uposition (gtkpasman1, xl, yl);

  /* then add hidden entry used to have paste selections */
  clipboard = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (vbox), clipboard, FALSE, TRUE, 0);
  gtk_widget_show (clipboard);
  gtk_widget_hide (clipboard);

  return gtkpasman1;
}

/*
 * create_gtkpasman_eye
 *      create the eye window and its menu
 */
GtkWidget *
create_gtkpasman_eye (void)
{
  GtkWidget *fixed;
  GdkPixbuf *off_pixbuf, *on_pixbuf, *scaled_off_pixbuf, *scaled_on_pixbuf;

  GtkAccelGroup *menu2_accels;

  /* main window */
  gtkpasman2 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (gtkpasman2), "gtkpasman");
  gtk_window_set_wmclass (GTK_WINDOW (gtkpasman2),
			  "gtkpasmanEye", "gtkpasman");
  gtk_signal_connect (GTK_OBJECT (gtkpasman2), "delete_event",
		      (GtkSignalFunc) gtk_main_quit, NULL);
  gtk_signal_connect (GTK_OBJECT (gtkpasman2), "destroy_event",
		      (GtkSignalFunc) gtk_main_quit, NULL);

  /* menus */
  menu2_accels = gtk_accel_group_new ();
  item_factory2 = gtk_item_factory_new (GTK_TYPE_MENU, "<Main>",
					menu2_accels);
  gtk_item_factory_create_items (item_factory2, 8, Menu2, NULL);
  gtk_accel_group_attach (menu2_accels, GTK_OBJECT (gtkpasman2));

  /* eye picture */
  off_pixbuf = gdk_pixbuf_new_from_xpm_data (off_xpm);
  scaled_off_pixbuf = gdk_pixbuf_scale_simple (off_pixbuf, zoomsize,
					       zoomsize, GDK_INTERP_NEAREST);
  gdk_pixbuf_render_pixmap_and_mask (scaled_off_pixbuf, &off, &mask_off, 127);

  on_pixbuf = gdk_pixbuf_new_from_xpm_data (on_xpm);
  scaled_on_pixbuf = gdk_pixbuf_scale_simple (on_pixbuf, zoomsize,
					      zoomsize, GDK_INTERP_NEAREST);
  gdk_pixbuf_render_pixmap_and_mask (scaled_on_pixbuf, &on, &mask_on, 127);

  current = gtk_pixmap_new (off, mask_off);
  gdk_pixbuf_unref (off_pixbuf);
  gdk_pixbuf_unref (on_pixbuf);
  gdk_pixbuf_unref (scaled_off_pixbuf);
  gdk_pixbuf_unref (scaled_on_pixbuf);
  gtk_widget_show (current);

  /* callback for clic */
  gtk_signal_connect (GTK_OBJECT (gtkpasman2), "button-press-event",
		      (GtkSignalFunc) eye_clic, GTK_OBJECT (item_factory2));

  /* fixed */
  fixed = gtk_fixed_new ();
  gtk_fixed_put (GTK_FIXED (fixed), current, 0, 0);
  gtk_widget_show (fixed);

  gtk_container_add (GTK_CONTAINER (gtkpasman2), fixed);

  gtk_widget_shape_combine_mask (gtkpasman2, mask_off, 0, 0);

  /* show all */
  gtk_widget_show_all (gtkpasman2);
  if ((xe != -1) && (ye != -1))
    gtk_widget_set_uposition (gtkpasman2, xe, ye);

  return gtkpasman2;
}

/*
 * create_gtkpasman_askpassword
 *      create the asking passphrase window
 */
void
create_gtkpasman_askpassword (void)
{
  GtkWidget *dialog;
  GtkWidget *passtxt;
  GtkWidget *okbutton;
  GtkWidget *cancelbutton;

  /* close the eye, empty tree and recent menu */
  gtk_pixmap_set (GTK_PIXMAP (current), off, mask_off);
  emptytree (tree1);
  empty_hosts_recent_menu ();

  /* main windows (modal) */
  dialog = gtk_dialog_new ();
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_window_set_title (GTK_WINDOW (dialog), "PGP Passphrase?");
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "ReadPassword", "gtkpasman");

  /* password entry */
  passtxt = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      passtxt, FALSE, FALSE, 0);
  gtk_entry_set_visibility (GTK_ENTRY (passtxt), FALSE);
  gtk_widget_grab_focus (passtxt);
  gtk_signal_connect_object (GTK_OBJECT (passtxt), "activate",
			     (GtkSignalFunc) change_pass,
			     GTK_OBJECT (passtxt));
  gtk_signal_connect_object (GTK_OBJECT (passtxt), "activate",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  /* cancel */
  cancelbutton = gtk_button_new_with_label ("Cancel");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      cancelbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  /* ok */
  okbutton = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      okbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (okbutton), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  /* show all */
  gtk_widget_show_all (dialog);
}

/*
 * update_positions
 *      retrieve window positions and store them in config file
 */
void
update_positions (int getfromscreen)
{
  if (getfromscreen) {
    gdk_window_get_position (gtkpasman1->window, &xl, &yl);
    gdk_window_get_position (gtkpasman2->window, &xe, &ye);
  }
  config_write ();
}

/*
 * update_display
 *      show or hide eye and list
 */
void
update_display (void)
{
  if (listvisible) {
    gtk_widget_show (gtkpasman1);
    gtk_widget_set_uposition (gtkpasman1, xl, yl);
    if (!eyevisible)
      gtk_widget_hide (gtkpasman2);
    else {
      /* if eye is always visible... */
      gtk_widget_show (gtkpasman2);
      gtk_widget_set_uposition (gtkpasman2, xe, ye);
    }
  }
  else {
    gtk_widget_hide (gtkpasman1);
    gtk_widget_show (gtkpasman2);
    gtk_widget_set_uposition (gtkpasman2, xe, ye);
  }
}

/*
 * change_display_state
 *      switch list shown/hidden and update display
 *      store positions of windows
 */
void
change_display_state (void)
{
  listvisible = !listvisible;
  update_positions (1);
  update_display ();
}

/*
 * eye_clic
 *      callback for eye clic
 *      popup menu for right clic
 */
int
eye_clic (GtkWidget * widget, GdkEventButton * event, gpointer data)
{
  if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
    change_display_state ();
  }
  else if (event->type == GDK_BUTTON_PRESS) {
    if (event->button == 1) {
      /* nothing (used to be free move in gpasman) */
    }
    else if (event->button == 3) {
      gtk_menu_popup (GTK_MENU
		      (gtk_item_factory_get_widget (data, "<Main>")),
		      NULL, NULL, NULL, NULL, 3, event->time);
    }
  }
  return 0;
}

/*
 * change_pass
 *      change the password and try to fill the tree
 */
void
change_pass (GtkWidget * passtxt)
{
  /* read password from the entry widget */
  globalpasswd = g_strdup (gtk_entry_get_text (GTK_ENTRY (passtxt)));

  /* close the eye, empty tree and recent menu */
  gtk_pixmap_set (GTK_PIXMAP (current), off, mask_off);
  emptytree (tree1);
  empty_hosts_recent_menu ();

  if ((decrypted_passwds = filltree (tree1)))
    /* if everything was correct, open the eye */
    gtk_pixmap_set (GTK_PIXMAP (current), on, mask_on);

  /* No need to keep passphrase in memory now... */
  g_free (globalpasswd);
}

/*
 * select_row
 *      keep track of the selected node
 */
void
select_row (GtkCTree * ctree, GtkCTreeNode * node)
{
  selectednode = node;
}

/*
 * select_command
 *      put command in the clipboard
 */
void
select_command (gchar ** data)
{
  gchar *txt;

  if (data != NULL) {
    txt = data[1];
    gtk_entry_set_text (GTK_ENTRY (clipboard), txt);
    gtk_editable_select_region (GTK_EDITABLE (clipboard), 0, -1);
    gtk_editable_copy_clipboard (GTK_EDITABLE (clipboard));
    add_to_recent_menu (data[2]);
  }
}

/*
 * select_passwd
 *      put password in the clipboard
 */
void
select_passwd (gchar ** data)
{
  gchar *txt;

  if (data != NULL) {
    txt = data[0];
    gtk_entry_set_text (GTK_ENTRY (clipboard), txt);
    gtk_editable_select_region (GTK_EDITABLE (clipboard), 0, -1);
    gtk_editable_copy_clipboard (GTK_EDITABLE (clipboard));
    add_to_recent_menu (data[2]);
  }
}

/*
 * select_passwd_and_exec_command
 *      put password in the clipboard
 *      execute command
 */
void
select_passwd_and_exec_command (gchar ** data)
{
  gchar *txt;

  select_passwd (data);
  if (data != NULL) {
    txt = data[1];
    system (txt);
  }
}

/*
 * button_copy_cmd_line
 *      callback for button
 */
void
button_copy_cmd_line (void)
{
  gchar **data;

  if (decrypted_passwds && selectednode) {
    data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1),
						   selectednode);
    select_command (data);
  }
}

/*
 * button_copy_passwd
 *      callback for button
 */
void
button_copy_passwd (void)
{
  gchar **data;

  if (decrypted_passwds && selectednode) {
    data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1),
						   selectednode);
    select_passwd (data);
  }
}

/*
 * button_copy_passwd_and_execute
 *      callback for button
 */
void
button_copy_passwd_and_execute (void)
{
  gchar **data;

  if (decrypted_passwds && selectednode) {
    data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1),
						   selectednode);
    select_passwd_and_exec_command (data);
  }
}

/*
 * menu_copy_passwd
 *      callback for menu
 */
void
menu_copy_passwd (gpointer data)
{
  if (decrypted_passwds)
    select_passwd (data);
}

/*
 * menu_copy_passwd_and_execute
 *      callback for menu
 */
void
menu_copy_passwd_and_execute (gpointer data)
{
  if (decrypted_passwds)
    select_passwd_and_exec_command (data);
}

/*
 * strip_slashes_and_underscores
 *      strip slashes and underscores...
 */
gchar *
strip_slashes_and_underscores (gchar * str)
{

  g_print ("%s\n", str);

  /* backslash slashes */

  /* backslash underscores */

  /* Free resources */
  return str;
}

/*
 * give_factory_path
 *      remove empty fields and build the apropriate factory path
 */
gchar *
give_factory_path (gchar * customer, gchar * group, gchar * host,
		   gchar * service, gchar * serviceplus, gchar * user)
{
  gchar *path1, *path2, *tmp;

  path1 = g_strconcat ("/Hosts/", customer, NULL);
  if (strcmp (group, "")) {
    path2 = g_strconcat (path1, "/", group, NULL);
    g_free (path1);
    path1 = g_strdup (path2);
    g_free (path2);
  }
  if (strcmp (host, "")) {
    path2 = g_strconcat (path1, "/", host, NULL);
    g_free (path1);
    path1 = g_strdup (path2);
    g_free (path2);
  }
  if (strcmp (service, "")) {
    path2 = g_strconcat (path1, "/", service, NULL);
    g_free (path1);
    path1 = g_strdup (path2);
    g_free (path2);
  }
  if (strcmp (serviceplus, "")) {
    path2 = g_strconcat (path1, "/", serviceplus, NULL);
    //tmp = strip_slashes_and_underscores(serviceplus);
    //path2 = g_strconcat(path1, "/", tmp, NULL);
    g_free (path1);
    path1 = g_strdup (path2);
    g_free (path2);
  }
  if (strcmp (user, "")) {
    path2 = g_strconcat (path1, "/", user, NULL);
  }
  else {
    path2 = g_strconcat (path1, "/(none)", NULL);
  }
  g_free (path1);

  return path2;
}

/*
 * give_factory_path_shortcut
 *      remove empty fields and build the apropriate factory path
 *      special handle for service only entries
 */
gchar *
give_factory_path_shortcut (gchar * customer, gchar * group, gchar * host,
			    gchar * servicecourt)
{
  gchar *path1, *path2;

  path1 = g_strconcat ("/Hosts/", customer, NULL);
  if (strcmp (group, "")) {
    path2 = g_strconcat (path1, "/", group, NULL);
    g_free (path1);
    path1 = g_strdup (path2);
    g_free (path2);
  }
  if (strcmp (host, "")) {
    path2 = g_strconcat (path1, "/", host, NULL);
    g_free (path1);
    path1 = g_strdup (path2);
    g_free (path2);
  }
  path2 = g_strconcat (path1, "/", servicecourt, NULL);
  g_free (path1);

  return path2;
}

/*
 * add_to_host_menu
 *      add an entry to the host menu
 */
void
add_to_host_menu (gchar * customer, gchar * group, gchar * host,
		  gchar * service, gchar * serviceplus, gchar * user,
		  gchar ** data)
{
  GtkItemFactoryEntry item;

  item.path = give_factory_path (customer, group, host, service,
				 serviceplus, user);
  item.accelerator = NULL;
  item.callback = &menu_copy_passwd_and_execute;
  item.callback_action = 0;
  item.item_type = "<Item>";
  gtk_item_factory_create_item (item_factory2, &item, data, 1);
  g_free (item.path);
}

/*
 * add_to_host_menu_sshroot
 *      add ssh+root entries to host menu
 */
void
add_to_host_menu_sshroot (gchar * customer, gchar * group, gchar * host,
			  gchar ** data)
{
  GtkItemFactoryEntry item;

  item.path = give_factory_path_shortcut (customer, group, host,
					  "Ssh root: copy passwd and execute");
  item.accelerator = NULL;
  item.callback = &menu_copy_passwd_and_execute;
  item.callback_action = 0;
  item.item_type = "<Item>";
  gtk_item_factory_create_item (item_factory2, &item, data, 1);
  g_free (item.path);

  item.path = give_factory_path_shortcut (customer, group, host,
					  "Ssh root: copy passwd");
  item.accelerator = NULL;
  item.callback = &menu_copy_passwd;
  item.callback_action = 0;
  item.item_type = "<Item>";
  gtk_item_factory_create_item (item_factory2, &item, data, 1);
  g_free (item.path);
}

/*
 * create_gtkpasman_config
 *      create the configuration window
 */
void
create_gtkpasman_config (void)
{
  GtkWidget *dialog;
  GtkWidget *frame1;
  GtkWidget *frame2;
  GtkWidget *frame3;
  GtkWidget *label4;
  GtkWidget *label5;
  GtkWidget *label6;
  GtkWidget *label7;
  GtkWidget *label8;
  GtkWidget *hbox2;
  GtkWidget *vbox2;
  GtkWidget *vbox3;
  GtkWidget *hbox3;
  GtkWidget *okbutton;
  GtkWidget *cancelbutton;
  gchar *tmp;

  /* main window (modal) */
  dialog = gtk_dialog_new ();
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_window_set_title (GTK_WINDOW (dialog), "Setup");
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "Setup", "gtkpasman");

  /* password file */
  frame1 = gtk_frame_new ("Password file");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      frame1, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame1), hbox2);
  label4 = gtk_label_new ("Default file");
  gtk_box_pack_start (GTK_BOX (hbox2), label4, FALSE, FALSE, 0);
  entry4 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry4), passwdfile);
  gtk_box_pack_start (GTK_BOX (hbox2), entry4, FALSE, FALSE, 0);
  /* TODO: browse files */
  /* button = gtk_button_new_with_label ("Browse");
     gtk_box_pack_start (GTK_BOX(hbox2),
     button, FALSE, FALSE, 0); */

  /* Command lines */
  frame2 = gtk_frame_new ("Commands");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      frame2, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame2), vbox2);

  label5 = gtk_label_new ("Ssh command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label5, FALSE, FALSE, 0);
  entry5 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry5), sshcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry5, FALSE, FALSE, 0);

  label6 = gtk_label_new ("Ftp command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label6, FALSE, FALSE, 0);
  entry6 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry6), ftpcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry6, FALSE, FALSE, 0);

  label7 = gtk_label_new ("Telnet command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label7, FALSE, FALSE, 0);
  entry7 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry7), telnetcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry7, FALSE, FALSE, 0);

  /* UI Behavior */
  frame3 = gtk_frame_new ("UI behavior");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      frame3, FALSE, FALSE, 0);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame3), vbox3);
  button4 = gtk_check_button_new_with_label ("Eye always visible?");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button4), eyevisible == 1);
  gtk_box_pack_start (GTK_BOX (vbox3), button4, FALSE, FALSE, 0);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (vbox3), hbox3);

  label8 = gtk_label_new ("Eye width:");
  gtk_box_pack_start (GTK_BOX (hbox3), label8, FALSE, FALSE, 0);
  entry8 = gtk_entry_new ();
  tmp = g_strdup_printf ("%d", zoomsize);
  gtk_entry_set_text (GTK_ENTRY (entry8), tmp);
  gtk_box_pack_start (GTK_BOX (hbox3), entry8, FALSE, FALSE, 0);
  g_free (tmp);

  button5 = gtk_check_button_new_with_label ("Remember positions?");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button5),
				rememberpos == 1);
  gtk_box_pack_start (GTK_BOX (vbox3), button5, FALSE, FALSE, 0);

  /* action area */
  cancelbutton = gtk_button_new_with_label ("Cancel");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      cancelbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  okbutton = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      okbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (okbutton), "clicked",
			     (GtkSignalFunc) change_config,
			     GTK_OBJECT (dialog));

  /* show all */
  gtk_widget_show_all (dialog);
}

/*
 * change_config
 *      store the values from the configuration window
 */
void
change_config (GtkWidget * dialog)
{
  int tmp;

  g_free (passwdfile);
  g_free (sshcmd);
  g_free (ftpcmd);
  g_free (telnetcmd);
  passwdfile = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry4)));
  sshcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry5)));
  ftpcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry6)));
  telnetcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry7)));
  eyevisible = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button4));
  rememberpos = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button5));
  if (sscanf (gtk_entry_get_text (GTK_ENTRY (entry8)), "%d", &tmp)) {
    zoomsize = tmp;
  }
  gtk_widget_destroy (dialog);
  update_display ();
  config_write ();
}

/*
 * add_to_last_menu
 *      add an entry the recent menu
 */
void
add_to_recent_menu (gchar * title)
{
  GtkItemFactoryEntry item;

  item.path = g_strdup_printf ("/Recent/%s", title);
  item.accelerator = NULL;
  item.callback = NULL;
  item.callback_action = 0;
  item.item_type = "<Item>";
  gtk_item_factory_create_item (item_factory2, &item, NULL, 1);
  g_free (item.path);
}

/*
 * empty_hosts_recent_menu
 *      delete hosts and recent menus and re-create them
 */
void
empty_hosts_recent_menu (void)
{
  GtkItemFactoryEntry item;

  /* delete */
  gtk_item_factory_delete_item (item_factory2, "/Hosts");
  gtk_item_factory_delete_item (item_factory2, "/Recent");
  /* re-create */
  item.path = "/Hosts";
  item.accelerator = NULL;
  item.callback = NULL;
  item.callback_action = 0;
  item.item_type = "<Branch>";
  gtk_item_factory_create_item (item_factory2, &item, NULL, 1);
  item.path = "/Recent";
  item.accelerator = NULL;
  item.callback = NULL;
  item.callback_action = 0;
  item.item_type = "<Branch>";
  gtk_item_factory_create_item (item_factory2, &item, NULL, 1);
}
