/*
 * gtkpasman
 * Copyright (C) 2003 Cyril Bellot
 *
 * main.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <gtk/gtk.h>
#include <unistd.h>
#include <stdlib.h>
#include "config.h"
#include "configread.h"
#include "interface.h"

/*
 * usage
 * 	display command line options
 */
void
usage (char *prg)
{
  g_print ("usage: %s [options] [password_file]\n"
	   "\n"
	   "valid options are:\n"
	   "  -g : Main window position <xoff>+<yoff>, like -g 100+20\n"
	   "  -e : Eye position <xoff>+<yoff>\n"
	   "  -s : Iconsize (width and height)\n"
	   "  -l : Invisible list (and visible eye)\n"
	   "  -a : Eye is always visible\n"
	   "  -n : Do not ask password at start\n", prg);

}

/*
 * main
 * 	parse arguments, call creations of windows and start the thing
 */
int
main (int argc, char *argv[])
{
  int c, initial_password = 1;
  char *p;

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  config_read ();
  /* parse arguments */
  while ((c = getopt (argc, argv, "g:e:s:lanh?")) != -1) {
    switch (c) {
      /* list position */
    case 'g':
      xl = strtol (optarg, &p, 10);
      if (*p == '+')
	yl = strtol (p + 1, NULL, 10);
      break;
      /* eye position */
    case 'e':
      xe = strtol (optarg, &p, 10);
      if (*p == '+')
	ye = strtol (p + 1, NULL, 10);
      break;
      /* zoom size */
    case 's':
      zoomsize = strtol (optarg, &p, 10);
      break;
      /* list invisible */
    case 'l':
      listvisible = 0;
      break;
      /* eye always visible */
    case 'a':
      eyevisible = 1;
      break;
      /* do not ask password at start */
    case 'n':
      initial_password = 0;
      break;
      /* help */
    case '?':
    case 'h':
      usage (argv[0]);
      return 0;
    }
  }
  /* password file from command line */
  if (argc - optind > 0) {
    g_free (passwdfile);
    passwdfile = g_strdup (argv[optind]);
  }

  /* Store user defined window positions (and zoom...) */
  update_positions (0);

  /* create windows */
  create_gtkpasman_list ();
  create_gtkpasman_eye ();

  update_display ();

  /* ask initial password */
  if (initial_password)
    create_gtkpasman_askpassword ();

  gtk_main ();

  /* Get window positions, store all settings */
  update_positions (1);
  return 0;
}
